import os
from ament_index_python.packages import get_package_share_directory
from launch_ros.actions import Node
from launch import LaunchDescription
from launch.actions import DeclareLaunchArgument, IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.substitutions import LaunchConfiguration

def generate_launch_description():
    use_sim_time = LaunchConfiguration('use_sim_time', default='false')

    jobot_nav_dir = get_package_share_directory('jobot_nav2')
    jobot_nav_launchr = os.path.join(jobot_nav_dir, 'launch')


    param_file_name = 'param_myagv_mec.yaml'
    param_dir = os.path.join(jobot_nav_dir, 'param','jobot_param')
    param_file = LaunchConfiguration('params', default=os.path.join(param_dir, param_file_name))

    return LaunchDescription([
        DeclareLaunchArgument(
            'map',
            default_value='',
            description='Full path to map file to load'),

        DeclareLaunchArgument(
            'params',
            default_value=param_file,
            description='Full path to param file to load'),
        Node(
            name='waypoint_cycle',
            package='nav2_waypoint_cycle',
            executable='nav2_waypoint_cycle',
        ),

        IncludeLaunchDescription(
            PythonLaunchDescriptionSource(
                [jobot_nav_launchr, '/bringup_launch_for_slam.py']),
            launch_arguments={
                'map': '',
                'slam': 'True',
                'use_sim_time': use_sim_time,
                'params_file': param_file}.items(),
        ),

    ])
