#ifndef MYAGVPi_H
#define MYAGVPi_H

#include <rclcpp/rclcpp.hpp>
#include <geometry_msgs/msg/twist.hpp>
#include <nav_msgs/msg/odometry.hpp>
#include <geometry_msgs/msg/transform_stamped.hpp>
#include <sensor_msgs/msg/imu.hpp>
#include <std_msgs/msg/float32.hpp>
#include <tf2/LinearMath/Quaternion.h>
#include <tf2_ros/transform_broadcaster.h>
#include <tf2/LinearMath/Quaternion.h>
#include <tf2_geometry_msgs/tf2_geometry_msgs.h>
#include <boost/asio.hpp>
#include <chrono>
#include <memory>
#include <boost/array.hpp>

#define twoKpDef	1.0f				// (2.0f * 0.5f)	// 2 * proportional gain
#define twoKiDef	0.0f				// (2.0f * 0.0f)	// 2 * integral gain
#define TOTAL_RECEIVE_SIZE 27         	// 27 字节 //The length of the data sent by the esp32 //esp32发送过来的数据的长度
#define OFFSET_COUNT 	200

class MyAGV : public rclcpp::Node{
public:
    MyAGV(const std::string &node_name = "myagv_odometry_node");
	~MyAGV();
	bool init();
	float invSqrt(float number);
	void execute(double linearX, double linearY, double angularZ);
	void MahonyAHRSupdateIMU(float gx, float gy, float gz, float ax, float ay, float az);
	void accelerometerOffset(float gx, float gy, float gz);
	void publisherOdom();
	void publisherImuSensor();
	void publisherImuSensorRaw();
	void Publish_Voltage();

private:
	bool readSpeed();
	void writeSpeed(double movex, double movey, double rot);
	void restore();
	void restoreRun();

	// void cmdVelCallback(const geometry_msgs::msg::Twist::SharedPtr msg);
	
	// std::shared_ptr<std::thread> read_data_thread_;
	// rclcpp::Rate loop_rate_;

	double x;
	double y;
	double theta;

	double vx;
	double vy;
	double vtheta;

	double ax;
	double ay;
	double az;

	double wx;
	double wy;
	double wz;

	float Gyroscope_Xdata_Offset;
	float Gyroscope_Ydata_Offset;
	float Gyroscope_Zdata_Offset;
	float sampleFreq;
	float Battery_voltage,Backup_Battery_voltage;
	unsigned short Offest_Count;

	bool pub_odom_tf_ = false;

    std::shared_ptr<rclcpp::Node> node_;
    rclcpp::Publisher<nav_msgs::msg::Odometry>::SharedPtr pub_odom;
    rclcpp::Publisher<std_msgs::msg::Float32>::SharedPtr pub_voltage;
    rclcpp::Publisher<sensor_msgs::msg::Imu>::SharedPtr pub_imu;

    std::unique_ptr<tf2_ros::TransformBroadcaster> odomBroadcaster;
	// rclcpp::Subscription<geometry_msgs::msg::Twist>::SharedPtr cmd_vel_subscription_;
    rclcpp::Time currentTime, lastTime;
    sensor_msgs::msg::Imu imu_data;
};

#endif // !MYAGVPi_H